function CopyTemperature(name, devTemperature) {
    dev[name + "/Temp"] = dev[devTemperature];
}

function PID(name) {
    var kp = dev[name + "/Kp"];
    var ki = dev[name + "/Ki"];
    var kd = dev[name + "/Kd"];

    var input = dev[name + "/Temp"];
    var error = dev[name + "/SP"] - input;	        //  

    var delta_error = error - dev[name + "/Prev"];	//     dt
    dev[name + "/Prev"] = error;					//  

    var integral = dev[name + "/IntErr"];

    var Dt = 1000;
    var Dt_s = Dt / 1000.0;

    integral += error * ki * Dt_s;
    dev[name + "/IntErr"] = integral;

    var active = dev[name + "/Active"];
    var output = error * kp + integral + delta_error * kd / Dt_s;

    if (active) {
        setTimeout(
            function () {
                PID(name);
            },
            Dt
        );
    } else {
        output = 0;
    }

    dev[name + "/Output"] = output;
}

exports.Add = function (name, title, devTemperature, onOutput) {
    defineVirtualDevice(
        name,
        {
            title: title,
            cells: {
                Active: {
                    type: "switch",
                    title: " ",
                    value: false,
                    readonly: false,
                    order: 1,
                },
                Temp: {
                    type: "temperature",
                    title: " ",
                    value: 0,
                    order: 2
                },
                SP: {
                    type: "temperature",
                    title: "",
                    value: 22,
                    order: 3,
                    readonly: false
                },
                Output: {
                    type: "value",
                    title: " ",
                    value: 0,
                    order: 4
                },
                Kp: {
                    type: "value",
                    title: " Kp",
                    value: 5.0,
                    order: 5,
                    readonly: false
                },
                Ki: {
                    type: "value",
                    title: " Ki",
                    value: 0.01,
                    order: 6,
                    readonly: false
                },
                Kd: {
                    type: "value",
                    title: " Kd",
                    value: 3.0,
                    order: 7,
                    readonly: false
                },
                Prev: {
                    type: "value",
                    title: " ",
                    value: 0.0,
                    order: 8,
                    readonly: true
                },
                IntErr: {
                    type: "value",
                    title: " ",
                    value: 0.0,
                    order: 9,
                    readonly: false
                }
            }
        }
    );

    var active = name + "/Active";

    defineRule(name + "Active", {
        whenChanged: [active],
        then: function () {
            if (dev[active]) {
                dev[name + "/IntErr"] = 0;
                dev[name + "/Prev"] = 0;
                PID(name);
            }
        }
    });

    CopyTemperature(name, devTemperature);

    defineRule(
        name + "TemperatureChanged",
        {
            whenChanged: [devTemperature],
            then: function () { CopyTemperature(name, devTemperature); }
        }
    );

    if ((onOutput !== undefined) && (onOutput != null)) {
        defineRule(
            name + "OutputChanged",
            {
                whenChanged: [name + "/Output"],
                then: function () {
                    var output = dev[name + "/Output"];
                    onOutput(output);
                }
            }
        );
    }
};